package cc.smtweb.system.bpm.web.engine.model.listcard.mastersub;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.jdbc.AbsDbWorker;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.model.listcard.single.LCSingleHelper;
import cc.smtweb.system.bpm.web.engine.model.listcard.single.LCSingleSaveHandler;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Set;

/**
 * @Author:lip
 * @Date : 2022/8/28 21:03
 */
public class LCMsSaveHandler extends LCSingleSaveHandler {
    private String cardMaster = "cardMaster";
    private String listSub = "listSub";

    /**
     * 保存指定数据集操作，入参：{pageId,dataset:"", data:{form:{},list: {total:0,rows:[]}}, filter:{}}
     */
    public R save() {
        //数据集
        String dbName = params.readString("dataset");
        SwMap filter = params.readMap("filter");

        //待保存数据
        SwMap data = params.readMap("data");
        if (data == null) throw new BizException("没有收到待保存的的数据：" + dbName + "！");

        String subForeignKey = form.getOpts().readMap("config").readMap("props").readString("subForeignKey","-1");
        Assert.isTrue(!subForeignKey.equals("-1"),"未获取到子表外键配置");

        //对应的数据集定义
        PageDataset masterDataSet = findDataset(cardMaster);
        PageDataset subDataSet = findDataset(listSub);
        Assert.isTrue(masterDataSet != null && subDataSet != null,"没有找到数据集");

        //读取待保存的bean
        DefaultEntity masterBean = readBeanFromPage(masterDataSet, data.readMap(cardMaster).readMap("form"));
        if (filter != null && masterBean.isNew()) {
            //有过滤条件，将关联的值设上
            setLinkValue(masterDataSet, masterBean, f -> filter.get(f.name));
        }
        // 读取子表数据
        SwMap details = params.readMap("details");
        Assert.notNull(details,"没有收到待保存的明细数据");
        ArrayList<DefaultEntity> inserted = new ArrayList<>();
        ArrayList<DefaultEntity> updated = new ArrayList<>();
        details.readListMap("inserted").forEach(row -> {
            SwMap rowData = new SwMap();
            rowData.putAll(row);
            DefaultEntity insertBean = readBeanFromPage(subDataSet, rowData);
            insertBean.put(subForeignKey,masterBean.getEntityId());
            inserted.add(insertBean);
        });
        details.readListMap("updated").forEach(row -> {
            SwMap rowData = new SwMap();
            rowData.putAll(row);
            DefaultEntity updateBean = readBeanFromPage(subDataSet, rowData);
            updated.add(updateBean);
        });
        Set<Long> deleted = details.readLongSet("deleted");

        //
        checkBean(masterBean);

        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                saveBean(masterBean);
                ModelTable subTable = ModelTableCache.getInstance().get(subDataSet.masterTable);
                Assert.notNull(subTable,"未获取到子表");

                EntityDao<DefaultEntity> dao = DbEngine.getInstance().findDao(subTable.getName());
                dao.batchInsertEntity(inserted);
                dao.batchUpdateEntity(updated);
                if (deleted != null){
                    dao.deleteEntity(deleted);
                }
            }

            @Override
            public void doAfterDbCommit() {
                afterCommit(masterBean);
            }

            @Override
            public void doAfterDbRollback() {
                afterRollback(masterBean);
            }
        });
        return R.success(LCSingleHelper.buildRetData(masterDataSet.name, masterBean.getData()));
    }

    /**
     * 从页面获取待保存的bean
     *
     * @param pageDataSet
     * @param data
     * @return
     */
    protected DefaultEntity readBeanFromPage(PageDataset pageDataSet, SwMap data) {
        ModelTable table = ModelTableCache.getInstance().get(pageDataSet.masterTable);
        if (table == null) throw new BizException("没有找到待保存的表定义：" + pageDataSet.name);

        long id = data.readLong(table.getIdField());

        EntityDao<DefaultEntity> dao = DbEngine.getInstance().findDao(table.getName());
        DefaultEntity bean;
        if (id <= 0L) {
            bean = dao.createBean();
            bean.setIsNew(true);
            //暂时不考虑list保存的情况
            bean.getData().putAll(data);
        } else {
            bean = dao.queryEntity(id);
            if (bean == null) {
                throw new BizException("没有找到待保存的记录：" + table.getName() + "." + id);
            }
            //暂时不考虑list保存的情况
            bean.getData().putAll(data);
            bean.setIsNew(false);
        }
        return bean;
    }

}
